#!/bin/sh

init(){
    fsr_docker_utils_sh="/opt/cyops/configs/scripts/docker/utils.sh"
    source $fsr_docker_utils_sh
}

create_docker_flag(){
    touch $fdc_flag_fsr_in_docker
    chmod 644 $fdc_flag_fsr_in_docker
}

is_fresh_install(){
    if [ -f "$fdc_flag_vm_configured" ]; then
        # 1 means false
        return 1
    fi
    # 0 means true
    return 0
}

#
# Purpose of this sub-routine is to run the tasks for first boot on docker, be it FMG or normal
# docker.
#
tasks_for_first_boot(){
    #
    # 1. Run the config wizard
    # 2. Update hostname and fqhn in database
    # 3. Configured the Embedded SME
    #

    #
    # Run the config VM. Config VM needs services like nginx, rabbitmq etc. to come up
    # before actual starting. Hence wait for 15 secs.
    #

    sleep 15

    /opt/cyops/configs/scripts/config-vm.sh --non-interactive


    # Updates hostname and fqhn in database
    tasks_for_every_boot_on_fmg

    #
    # Enable the embedded SME
    #
    # For docker, the TCP port will always be 55671. This is handled in codebase.
    # And password for GUI is changeme.
    #

    csadm secure-message-exchange enable --no-interaction

}

tasks_for_first_boot_on_fmg(){
    #
    # Deploy extension license on fsr on fmg
    #

    if is_fmg; then
      /opt/cyops-auth/utilities/licensemanager.py --deploy-extension-license
    fi
}

tasks_for_every_boot_on_fmg(){

    hostname_fmg=$(get_hostname_fmg)

    if [ "X$hostname_fmg" = "X" ]; then
        echo "ERROR: Failed to find the FMG hostname"
        return
    fi

    Server_fqhn="${hostname_fmg}/${fdc_fmg_fsr_uri_start_keyword}"

    echo "Updating the server FQDN for workflow as [${Server_fqhn}]"

    psql -U cyberpgsql sealab -c "update dynamic_variable_dynamicvariable set value = '$Server_fqhn' where name='Server_fqhn'"

    crudhub_global_hostname="${hostname_fmg}/${fdc_fmg_fsr_uri_start_keyword}"

    echo "Updating the hostname in system settings as [$crudhub_global_hostname]"

    /opt/cyops/configs/scripts/api_caller.py \
               --endpoint "https://localhost/api/3/system_settings/845c05cc-05b3-450e-9afb-df6b6e436321" \
               --method PUT --payload "{\"globalValues\": { \"hostname\": \"$crudhub_global_hostname\"}}" >/dev/null
}

clean_up(){
    echo "Cleaning directory [$fdc_dir_backup]"
    rm -rf $fdc_dir_backup
}

is_license_installed(){
    local cmd_license_check="csadm license --show-details"
    local out=$($cmd_license_check)
    #
    # If license is not found, we get below msg:
    # License not found on the system.
    # We are checking for only 'not found on'
    #
    if echo $out | grep "not found on"; then
        # 1 means false
        return 1
    fi
    # 0 means true
    return 0
}

perform_post_license_deployment_steps(){
    echo "Performing post license deployment steps"
    ln -sf /opt/cyops-routing-agent/postman/postman/postman_wsgi.ini /etc/uwsgi.d/postman_wsgi.ini
    systemctl enable cyops-postman.service cyops-integrations-agent.service
    systemctl restart cyops-postman.service cyops-integrations-agent.service
}

perform_config_wizard_steps(){
    echo "Enabling and starting the HA service"
    systemctl enable cyops-ha.service
    systemctl start cyops-ha.service
}

setup_cron_for_periodic_backup(){
    #
    # Set cron job to take backup of configuration after every 1 hour
    #
    echo "0 */1 * * * root sh $fdc_cron_config_handler" > /etc/cron.d/cron-config-handler
}

clean_up_last_restored_backup(){
    #
    # when backup is restored, it left the extracted directory. for eg.
    # [root@6d551f3d7288 .do-not-delete-backups]# ls -d */
    # 20201022_121047
    # clean those
    #

    local dirs_to_clean=$(ls -d $fdc_dir_backup/*/)
    for dir_to_clean in $dirs_to_clean
    do
        if [ -d $dir_to_clean ]; then
            rm -rf $dir_to_clean
        fi
    done
}

handle_first_boot(){
    clean_up
    tasks_for_first_boot
    if [ -f "$FILE_FSR_RUNNING_IN_FMG" ]; then
        tasks_for_first_boot_on_fmg
    fi
}

handle_subsequent_boot(){
    perform_config_wizard_steps

    if is_license_installed; then
        perform_post_license_deployment_steps
    fi

    if [ -f "$FILE_FSR_RUNNING_IN_FMG" ]; then
        tasks_for_every_boot_on_fmg
    fi

    clean_up_last_restored_backup

    local last_backup=$(ls -trh $fdc_dir_backup/DR_BACKUP_*.tgz | tail -1)

    if [ "X$last_backup" = "X" ]; then
        return
    fi

    echo "Restoring the backup [$last_backup]"
    csadm db --restore $last_backup
}

my_exit(){
    local exit_status=$1
    mkdir -p $fdc_dir_backup
    csadm db --backup-config $fdc_dir_backup
    setup_cron_for_periodic_backup
    echo "Exiting the boot-config.sh with exit code as [$exit_status]"
    exit $exit_status
}

##################
# Main/main/Main starts here
##############

echo "Starting the boot-config.sh"

set +e

init

create_docker_flag

#
# When FSR docker runs in non FMG, below script may throw some error,
# which is expected.
#

$fdc_fmg_settings --set-timezone --create-flag

if [ -f "$FILE_FSR_RUNNING_IN_FMG" ]; then

    initialize_global_variables

    add_self_entry_in_etc_hosts

    if is_fresh_install; then
        notify_ui_fortisoar_state "false" "init"
    else
        notify_ui_fortisoar_state "false" "restore"
    fi

    sed -i 's#<base href="/">#<base href="/fortisoar/">#g' /opt/cyops-ui/cindex.html
fi

if is_fresh_install; then
    handle_first_boot
else
    handle_subsequent_boot
fi

if [ -f "$FILE_FSR_RUNNING_IN_FMG" ]; then
    notify_ui_fortisoar_state "true" "provisioned"
fi

my_exit 0